# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import copy
import json
import os
import sys

from collections import defaultdict
from os.path import isdir, join

import requests
import tqdm

from packaging.version import parse as parse_version

BASE_URL = "http://www.idiap.ch/software/bob/conda"
SUBDIRS = (
    "noarch",
    "linux-64",
    "osx-64",
)

REMOVALS = {
    "osx-64": {
        "aniso8601-3.0.0-py27haa2c913_0.tar.bz2",
        "aniso8601-3.0.0-py36haa2c913_0.tar.bz2",
        "aniso8601-4.0.1-py36h6217847_0.tar.bz2",
        "aniso8601-4.0.1-py36h8c28fb6_0.conda",
        "aniso8601-4.0.1-py37h8c28fb6_0.conda",
        "aniso8601-4.0.1-py37h8c28fb6_0.tar.bz2",
        "aspy.yaml-1.1.1-py36h6217847_0.tar.bz2",
        "aspy.yaml-1.1.1-py36h8c28fb6_0.conda",
        "aspy.yaml-1.1.1-py37h8c28fb6_0.conda",
        "aspy.yaml-1.1.1-py37h8c28fb6_0.tar.bz2",
        "cfgv-1.4.0-py36_0.conda",
        "cfgv-1.4.0-py36h493f39d_0.tar.bz2",
        "cfgv-1.4.0-py37_0.conda",
        "cfgv-1.4.0-py37h81a5984_0.tar.bz2",
        "colorlog-3.1.4-py27haf68d3b_0.tar.bz2",
        "colorlog-3.1.4-py36haf68d3b_0.tar.bz2",
        "cyvlfeat-0.4.5-np111py27_0.tar.bz2",
        "cyvlfeat-0.4.5-np111py27_1.tar.bz2",
        "cyvlfeat-0.4.5-np111py27_2.tar.bz2",
        "cyvlfeat-0.4.5-np111py34_0.tar.bz2",
        "cyvlfeat-0.4.5-np111py34_1.tar.bz2",
        "cyvlfeat-0.4.5-np111py34_2.tar.bz2",
        "cyvlfeat-0.4.5-np111py35_0.tar.bz2",
        "cyvlfeat-0.4.5-np111py35_1.tar.bz2",
        "cyvlfeat-0.4.5-np111py35_2.tar.bz2",
        "cyvlfeat-0.4.5-np111py36_2.tar.bz2",
        "cyvlfeat-0.4.5-np112py27_2.tar.bz2",
        "cyvlfeat-0.4.5-np112py35_2.tar.bz2",
        "cyvlfeat-0.4.5-np112py36_2.tar.bz2",
        "cyvlfeat-0.4.5-np113py27_2.tar.bz2",
        "cyvlfeat-0.4.5-np113py35_2.tar.bz2",
        "cyvlfeat-0.4.5-np113py36_2.tar.bz2",
        "cyvlfeat-0.4.6-np112py27_2.tar.bz2",
        "cyvlfeat-0.4.6-np112py35_2.tar.bz2",
        "cyvlfeat-0.4.6-np112py36_2.tar.bz2",
        "cyvlfeat-0.4.6-np113py27_2.tar.bz2",
        "cyvlfeat-0.4.6-np113py35_2.tar.bz2",
        "cyvlfeat-0.4.6-np113py36_2.tar.bz2",
        "cyvlfeat-0.4.6-py27hbf625f4_4.tar.bz2",
        "cyvlfeat-0.4.6-py27hd91dd34_3.tar.bz2",
        "cyvlfeat-0.4.6-py36hbf625f4_4.tar.bz2",
        "cyvlfeat-0.4.6-py36hd91dd34_3.tar.bz2",
        "cyvlfeat-0.4.6-py37hd4ba1bb_4.tar.bz2",
        "cyvlfeat-0.5.1-py36h490deae_0.tar.bz2",
        "cyvlfeat-0.5.1-py36h490deae_1.conda",
        "cyvlfeat-0.5.1-py37h490deae_0.tar.bz2",
        "cyvlfeat-0.5.1-py37h490deae_1.conda",
        "dataclasses-0.8-py36_0.conda",
        "dataclasses-0.8-py37_0.conda",
        "dataclasses-0.8-py37hacc1119_1.conda",
        "dataclasses-0.8-py38hacc1119_0.conda",
        "dataclasses-0.8-py38hacc1119_1.conda",
        "ddt-1.2.1-py36h8c28fb6_1.conda",
        "ddt-1.2.1-py36he61d863_2.conda",
        "ddt-1.2.1-py37h8c28fb6_1.conda",
        "ddt-1.2.1-py37he61d863_2.conda",
        "ddt-1.2.1-py38he61d863_2.conda",
        "dlib-19.0-np111py27_0.tar.bz2",
        "dlib-19.0-np111py27_1.tar.bz2",
        "dlib-19.0-np111py35_0.tar.bz2",
        "dlib-19.0-np111py35_1.tar.bz2",
        "dlib-19.0-np111py36_0.tar.bz2",
        "dlib-19.0-np111py36_1.tar.bz2",
        "dlib-19.0-np112py27_0.tar.bz2",
        "dlib-19.0-np112py27_1.tar.bz2",
        "dlib-19.0-np112py35_0.tar.bz2",
        "dlib-19.0-np112py35_1.tar.bz2",
        "dlib-19.0-np112py36_0.tar.bz2",
        "dlib-19.0-np112py36_1.tar.bz2",
        "dlib-19.0-np113py27_1.tar.bz2",
        "dlib-19.0-np113py35_1.tar.bz2",
        "dlib-19.0-np113py36_1.tar.bz2",
        "dlib-19.7-py27h080864a_1.tar.bz2",
        "dlib-19.7-py27h2be14d3_0.tar.bz2",
        "dlib-19.7-py36h080864a_1.tar.bz2",
        "dlib-19.7-py36h2be14d3_0.tar.bz2",
        "dlib-19.7-py36h371aa55_2.conda",
        "dlib-19.7-py36hae06d23_3.conda",
        "dlib-19.7-py36hc3aa795_2.tar.bz2",
        "dlib-19.7-py37h371aa55_2.conda",
        "dlib-19.7-py37h56cabac_2.tar.bz2",
        "dlib-19.7-py37hae06d23_3.conda",
        "ffmpeg-2.8.10-0.tar.bz2",
        "ffmpeg-2.8.10-1.tar.bz2",
        "ffmpeg-2.8.10-2.tar.bz2",
        "ffmpeg-2.8.10-3.tar.bz2",
        "ffmpeg-2.8.10-4.tar.bz2",
        "ffmpeg-3.4-h3b64a03_1.tar.bz2",
        "ffmpeg-4.0-h18e5fc6_2.conda",
        "ffmpeg-4.0-h2047f9e_1.tar.bz2",
        "ffmpeg-4.0-he86247c_0.tar.bz2",
        "flask-restful-0.3.6-py27haa2c913_0.tar.bz2",
        "flask-restful-0.3.6-py36h8c28fb6_0.conda",
        "flask-restful-0.3.6-py36haa2c913_0.tar.bz2",
        "flask-restful-0.3.6-py37h4c55d99_0.tar.bz2",
        "flask-restful-0.3.6-py37h8c28fb6_0.conda",
        "identify-1.1.8-py36_0.conda",
        "identify-1.1.8-py36_0.tar.bz2",
        "identify-1.1.8-py37_0.conda",
        "identify-1.1.8-py37_0.tar.bz2",
        "importlib_resources-1.0.2-py36_0.conda",
        "importlib_resources-1.0.2-py36_0.tar.bz2",
        "importlib_resources-1.0.2-py37_0.conda",
        "importlib_resources-1.0.2-py37_0.tar.bz2",
        "ipdb-0.10.2-py27_0.tar.bz2",
        "ipdb-0.10.2-py34_0.tar.bz2",
        "ipdb-0.10.2-py35_0.tar.bz2",
        "ipdb-0.10.2-py36_0.tar.bz2",
        "ipdb-0.10.3-py27h6a1c9b6_0.tar.bz2",
        "ipdb-0.10.3-py36h6a1c9b6_0.tar.bz2",
        "ipdb-0.11-py36h30e596e_0.tar.bz2",
        "ipdb-0.11-py37hbf0455a_0.tar.bz2",
        "ipdb-0.12.2-py36h8c28fb6_0.tar.bz2",
        "ipdb-0.12.2-py37h8c28fb6_0.tar.bz2",
        "ipdb-0.12.3-py36h8c28fb6_0.conda",
        "ipdb-0.12.3-py37h8c28fb6_0.conda",
        "ipdb-0.13.2-py36h8c28fb6_0.conda",
        "ipdb-0.13.2-py37h8c28fb6_0.conda",
        "ipdb-0.13.3-py36h8c28fb6_0.conda",
        "ipdb-0.13.3-py37h8c28fb6_0.conda",
        "libblitz-0.10-0.tar.bz2",
        "libblitz-0.10-1.tar.bz2",
        "libblitz-0.10-2.tar.bz2",
        "libblitz-0.10-3.tar.bz2",
        "libblitz-1.0.1-h0a44026_1.conda",
        "libblitz-1.0.1-hd7a9176_0.tar.bz2",
        "libmatio-1.5.11-h170b5ee_0.tar.bz2",
        "libmatio-1.5.11-hfe070ab_1.tar.bz2",
        "libmatio-1.5.15-h3257b41_0.tar.bz2",
        "libmatio-1.5.15-hf97b052_0.tar.bz2",
        "libmatio-1.5.15-hfe070ab_0.tar.bz2",
        "libmatio-1.5.17-hf97b052_0.conda",
        "libmatio-1.5.17-hfe070ab_0.conda",
        "libmatio-1.5.6-0.tar.bz2",
        "libmatio-1.5.6-1.tar.bz2",
        "libmatio-1.5.6-2.tar.bz2",
        "libmatio-1.5.6-3.tar.bz2",
        "libopencv-4.5.0-py37_0.conda",
        "libopencv-4.5.0-py37_1.conda",
        "libopencv-4.5.0-py38_1.conda",
        "libsvm-3.21-0.tar.bz2",
        "libsvm-3.21-1.tar.bz2",
        "libsvm-3.21-2.tar.bz2",
        "libsvm-3.21-3.tar.bz2",
        "libsvm-3.22-h095a671_0.tar.bz2",
        "libsvm-3.24-h0a44026_0.conda",
        "madmom-0.15.1-py27_0.tar.bz2",
        "madmom-0.15.1-py27h8174cad_1.tar.bz2",
        "madmom-0.15.1-py27hdf8f032_2.tar.bz2",
        "madmom-0.15.1-py35_0.tar.bz2",
        "madmom-0.15.1-py36_0.tar.bz2",
        "madmom-0.15.1-py36h8174cad_1.tar.bz2",
        "madmom-0.15.1-py36hdf8f032_2.tar.bz2",
        "madmom-0.16.1-py36hb0469d8_0.tar.bz2",
        "madmom-0.16.1-py37hb0469d8_0.tar.bz2",
        "menpo-0.7.7-np111py27_0.tar.bz2",
        "menpo-0.7.7-np111py27_1.tar.bz2",
        "menpo-0.7.7-np111py27_2.tar.bz2",
        "menpo-0.7.7-np111py27_3.tar.bz2",
        "menpo-0.7.7-np111py27_4.tar.bz2",
        "menpo-0.7.7-np111py34_0.tar.bz2",
        "menpo-0.7.7-np111py34_1.tar.bz2",
        "menpo-0.7.7-np111py34_2.tar.bz2",
        "menpo-0.7.7-np111py34_3.tar.bz2",
        "menpo-0.7.7-np111py34_4.tar.bz2",
        "menpo-0.7.7-np111py35_0.tar.bz2",
        "menpo-0.7.7-np111py35_1.tar.bz2",
        "menpo-0.7.7-np111py35_2.tar.bz2",
        "menpo-0.7.7-np111py35_3.tar.bz2",
        "menpo-0.7.7-np111py35_4.tar.bz2",
        "menpo-0.7.7-np111py36_4.tar.bz2",
        "menpo-0.7.7-np112py27_4.tar.bz2",
        "menpo-0.7.7-np112py35_4.tar.bz2",
        "menpo-0.7.7-np112py36_4.tar.bz2",
        "menpo-0.7.7-np113py27_4.tar.bz2",
        "menpo-0.7.7-np113py35_4.tar.bz2",
        "menpo-0.7.7-np113py36_4.tar.bz2",
        "menpo-0.8.1-np112py27_0.tar.bz2",
        "menpo-0.8.1-np112py35_0.tar.bz2",
        "menpo-0.8.1-np112py36_0.tar.bz2",
        "menpo-0.8.1-np113py27_0.tar.bz2",
        "menpo-0.8.1-np113py35_0.tar.bz2",
        "menpo-0.8.1-np113py36_0.tar.bz2",
        "menpo-0.8.1-py27h380936f_6.tar.bz2",
        "menpo-0.8.1-py27h7796296_3.tar.bz2",
        "menpo-0.8.1-py27h7c3da35_4.tar.bz2",
        "menpo-0.8.1-py27h7c3da35_5.tar.bz2",
        "menpo-0.8.1-py36h0057b77_7.tar.bz2",
        "menpo-0.8.1-py36h380936f_6.tar.bz2",
        "menpo-0.8.1-py36h7796296_3.tar.bz2",
        "menpo-0.8.1-py36h7c3da35_4.tar.bz2",
        "menpo-0.8.1-py36h7c3da35_5.tar.bz2",
        "menpo-0.8.1-py36ha1ea0de_8.tar.bz2",
        "menpo-0.8.1-py37ha1ea0de_8.tar.bz2",
        "menpo-0.9.1-py36h93b22a6_0.tar.bz2",
        "menpo-0.9.1-py36h93b22a6_1.tar.bz2",
        "menpo-0.9.1-py37h93b22a6_0.tar.bz2",
        "menpo-0.9.1-py37h93b22a6_1.tar.bz2",
        "menpo-0.9.2-py36h93b22a6_0.conda",
        "menpo-0.9.2-py37h93b22a6_0.conda",
        "menpofit-0.4.1-py27_0.tar.bz2",
        "menpofit-0.4.1-py27_1.tar.bz2",
        "menpofit-0.4.1-py34_0.tar.bz2",
        "menpofit-0.4.1-py34_1.tar.bz2",
        "menpofit-0.4.1-py35_0.tar.bz2",
        "menpofit-0.4.1-py35_1.tar.bz2",
        "menpofit-0.4.1-py36_1.tar.bz2",
        "menpofit-0.5.0-py27_0.tar.bz2",
        "menpofit-0.5.0-py35_0.tar.bz2",
        "menpofit-0.5.0-py36_0.tar.bz2",
        "menpofit-0.5.0-py37hbf0455a_0.tar.bz2",
        "menpofit-0.6.1-py36h8c28fb6_0.tar.bz2",
        "menpofit-0.6.1-py36h8c28fb6_1.tar.bz2",
        "menpofit-0.6.1-py36h8c28fb6_2.conda",
        "menpofit-0.6.1-py37h8c28fb6_0.tar.bz2",
        "menpofit-0.6.1-py37h8c28fb6_1.tar.bz2",
        "menpofit-0.6.1-py37h8c28fb6_2.conda",
        "menpowidgets-0.2.1p3-py27_0.tar.bz2",
        "menpowidgets-0.2.1p3-py34_0.tar.bz2",
        "menpowidgets-0.2.1p3-py35_0.tar.bz2",
        "menpowidgets-0.2.1p3-py36_0.tar.bz2",
        "menpowidgets-0.3.0p6-py27_0.tar.bz2",
        "menpowidgets-0.3.0p6-py35_0.tar.bz2",
        "menpowidgets-0.3.0p6-py36_0.tar.bz2",
        "mido-1.2.9-py36h8c28fb6_0.tar.bz2",
        "mido-1.2.9-py37h8c28fb6_0.tar.bz2",
        "mne-0.13.1-np111py27_0.tar.bz2",
        "mne-0.13.1-np111py34_0.tar.bz2",
        "mne-0.13.1-np111py35_0.tar.bz2",
        "mne-0.13.1-np111py36_0.tar.bz2",
        "mne-0.13.1-np112py27_0.tar.bz2",
        "mne-0.13.1-np112py35_0.tar.bz2",
        "mne-0.13.1-np112py36_0.tar.bz2",
        "mne-0.13.1-np113py27_0.tar.bz2",
        "mne-0.13.1-np113py35_0.tar.bz2",
        "mne-0.13.1-np113py36_0.tar.bz2",
        "mne-0.14.1-np112py27_0.tar.bz2",
        "mne-0.14.1-np112py35_0.tar.bz2",
        "mne-0.14.1-np112py36_0.tar.bz2",
        "mne-0.14.1-np113py27_0.tar.bz2",
        "mne-0.14.1-np113py35_0.tar.bz2",
        "mne-0.14.1-np113py36_0.tar.bz2",
        "mne-0.15.2-py27h249d46d_0.tar.bz2",
        "mne-0.15.2-py36h249d46d_0.tar.bz2",
        "mne-0.15.2-py37he5954f6_0.tar.bz2",
        "mne-0.18.1-py36h8c28fb6_0.conda",
        "mne-0.18.1-py36h8c28fb6_0.tar.bz2",
        "mne-0.18.1-py37h8c28fb6_0.conda",
        "mne-0.18.1-py37h8c28fb6_0.tar.bz2",
        "mr.developer-1.38-py27_0.tar.bz2",
        "mr.developer-1.38-py35_0.tar.bz2",
        "mr.developer-1.38-py36_0.tar.bz2",
        "mr.developer-1.38-py37h8c28fb6_0.tar.bz2",
        "mr.developer-2.0.0-py36h8c28fb6_0.conda",
        "mr.developer-2.0.0-py36h8c28fb6_0.tar.bz2",
        "mr.developer-2.0.0-py37h8c28fb6_0.conda",
        "mr.developer-2.0.0-py37h8c28fb6_0.tar.bz2",
        "nodeenv-1.3.3-py36h6217847_0.tar.bz2",
        "nodeenv-1.3.3-py36h8c28fb6_0.conda",
        "nodeenv-1.3.3-py37h8c28fb6_0.conda",
        "nodeenv-1.3.3-py37h8c28fb6_0.tar.bz2",
        "opencv-4.5.0-py37_0.conda",
        "opencv-4.5.0-py37_1.conda",
        "opencv-4.5.0-py38_1.conda",
        "openh264-1.7.0-h18e5fc6_2.conda",
        "openh264-1.7.0-h89e8454_1.tar.bz2",
        "oset-0.1.3-py27haf68d3b_0.tar.bz2",
        "oset-0.1.3-py36haf68d3b_0.tar.bz2",
        "oset-0.1.3-py37h3fd5425_0.tar.bz2",
        "pkgtools-0.7.3-py27_0.tar.bz2",
        "pkgtools-0.7.3-py34_0.tar.bz2",
        "pkgtools-0.7.3-py35_0.tar.bz2",
        "pkgtools-0.7.3-py36_0.tar.bz2",
        "pre_commit-1.13.0-py36_0.tar.bz2",
        "pre_commit-1.13.0-py36_1.conda",
        "pre_commit-1.13.0-py36_1.tar.bz2",
        "pre_commit-1.13.0-py37_0.tar.bz2",
        "pre_commit-1.13.0-py37_1.conda",
        "pre_commit-1.13.0-py37_1.tar.bz2",
        "py-opencv-4.5.0-py37hcf68f36_0.conda",
        "py-opencv-4.5.0-py37hcf68f36_1.conda",
        "py-opencv-4.5.0-py38ha368b55_1.conda",
        "pyedflib-0.1.11-py27h214fae5_0.tar.bz2",
        "pyedflib-0.1.11-py27hd741d17_1.tar.bz2",
        "pyedflib-0.1.11-py36h214fae5_0.tar.bz2",
        "pyedflib-0.1.11-py36hd741d17_1.tar.bz2",
        "pyedflib-0.1.11-py37hb44f9af_1.tar.bz2",
        "pyedflib-0.1.14-py36h1b85e82_0.conda",
        "pyedflib-0.1.14-py36h1b85e82_0.tar.bz2",
        "pyedflib-0.1.14-py37h1b85e82_0.conda",
        "pyedflib-0.1.14-py37h1b85e82_0.tar.bz2",
        "pyedflib-0.1.6-np111py27_0.tar.bz2",
        "pyedflib-0.1.6-np111py27_1.tar.bz2",
        "pyedflib-0.1.6-np111py27_2.tar.bz2",
        "pyedflib-0.1.6-np111py34_0.tar.bz2",
        "pyedflib-0.1.6-np111py34_1.tar.bz2",
        "pyedflib-0.1.6-np111py34_2.tar.bz2",
        "pyedflib-0.1.6-np111py35_0.tar.bz2",
        "pyedflib-0.1.6-np111py35_1.tar.bz2",
        "pyedflib-0.1.6-np111py35_2.tar.bz2",
        "pyedflib-0.1.6-np111py36_2.tar.bz2",
        "pyedflib-0.1.6-np112py27_2.tar.bz2",
        "pyedflib-0.1.6-np112py35_2.tar.bz2",
        "pyedflib-0.1.6-np112py36_2.tar.bz2",
        "pyedflib-0.1.6-np113py27_2.tar.bz2",
        "pyedflib-0.1.6-np113py35_2.tar.bz2",
        "pyedflib-0.1.6-np113py36_2.tar.bz2",
        "pyedflib-0.1.9-np112py27_0.tar.bz2",
        "pyedflib-0.1.9-np112py35_0.tar.bz2",
        "pyedflib-0.1.9-np112py36_0.tar.bz2",
        "pyedflib-0.1.9-np113py27_0.tar.bz2",
        "pyedflib-0.1.9-np113py35_0.tar.bz2",
        "pyedflib-0.1.9-np113py36_0.tar.bz2",
        "pytables-3.6.1-py37h6dbbf28_0.conda",
        "pytables-3.6.1-py38h6dbbf28_0.conda",
        "pytest-qt-3.2.2-py36h6217847_0.tar.bz2",
        "pytest-qt-3.2.2-py36h8c28fb6_0.conda",
        "pytest-qt-3.2.2-py36he61d863_1.conda",
        "pytest-qt-3.2.2-py37h8c28fb6_0.conda",
        "pytest-qt-3.2.2-py37h8c28fb6_0.tar.bz2",
        "pytest-qt-3.2.2-py37he61d863_1.conda",
        "pytest-qt-3.2.2-py38he61d863_1.conda",
        "python-gitlab-1.10.0-py36_0.tar.bz2",
        "python-gitlab-1.10.0-py37_0.tar.bz2",
        "python-gitlab-1.12.1-py36_0.conda",
        "python-gitlab-1.12.1-py37_0.conda",
        "python-gitlab-1.15.0-py36_0.conda",
        "python-gitlab-1.15.0-py37_0.conda",
        "python-gitlab-1.7.0-py36_0.tar.bz2",
        "python-gitlab-1.7.0-py36_1.tar.bz2",
        "python-gitlab-1.7.0-py37_1.tar.bz2",
        "python-gitlab-2.1.2-py36_0.conda",
        "python-gitlab-2.1.2-py37_0.conda",
        "python-gitlab-2.2.0-py36_0.conda",
        "python-gitlab-2.2.0-py37_0.conda",
        "python-gitlab-2.2.0-py38_0.conda",
        "python-gitlab-2.2.0-py39_0.conda",
        "python-gitlab-2.6.0-py37_0.conda",
        "python-gitlab-2.6.0-py38_0.conda",
        "python-gitlab-2.6.0-py39_0.conda",
        "pytorch-1.7.0-py3.6_0.tar.bz2",
        "pytorch-1.7.0-py3.7_0.tar.bz2",
        "pytorch-1.7.0-py3.8_0.tar.bz2",
        "schema-0.6.5-py27_0.tar.bz2",
        "schema-0.6.5-py34_0.tar.bz2",
        "schema-0.6.5-py35_0.tar.bz2",
        "schema-0.6.5-py36_0.tar.bz2",
        "schema-0.6.7-py27h91e9215_0.tar.bz2",
        "schema-0.6.7-py36h91e9215_0.tar.bz2",
        "schema-0.6.8-py36h426cc52_0.tar.bz2",
        "schema-0.6.8-py37he4c3ae9_0.tar.bz2",
        "schema-0.7.0-py36h8c28fb6_0.tar.bz2",
        "schema-0.7.0-py36h8c28fb6_1.tar.bz2",
        "schema-0.7.0-py37h8c28fb6_0.tar.bz2",
        "schema-0.7.0-py37h8c28fb6_1.tar.bz2",
        "schema-0.7.1-py36h8c28fb6_0.conda",
        "schema-0.7.1-py37h8c28fb6_0.conda",
        "schema-0.7.2-py36he61d863_0.conda",
        "schema-0.7.2-py37he61d863_0.conda",
        "schema-0.7.2-py38he61d863_0.conda",
        "schema-0.7.4-py37hd0a1e12_0.conda",
        "schema-0.7.4-py38hd0a1e12_0.conda",
        "sox-14.4.2-0.tar.bz2",
        "sox-14.4.2-1.tar.bz2",
        "sox-14.4.2-2.tar.bz2",
        "sox-14.4.2-3.tar.bz2",
        "sox-14.4.2-4.tar.bz2",
        "sox-14.4.2-5.tar.bz2",
        "sox-14.4.2-h4ff78cd_7.tar.bz2",
        "sox-14.4.2-hd88cb93_8.conda",
        "speex-1.2.0-0.tar.bz2",
        "sphinxcontrib-httpdomain-1.7.0-py36_0.tar.bz2",
        "sphinxcontrib-httpdomain-1.7.0-py36_1.conda",
        "sphinxcontrib-httpdomain-1.7.0-py36_2.conda",
        "sphinxcontrib-httpdomain-1.7.0-py37_0.tar.bz2",
        "sphinxcontrib-httpdomain-1.7.0-py37_1.conda",
        "sphinxcontrib-httpdomain-1.7.0-py37_2.conda",
        "sphinxcontrib-httpdomain-1.7.0-py38_2.conda",
        "sphinxcontrib-mermaid-0.6.1-py37hacc1119_0.conda",
        "sphinxcontrib-mermaid-0.6.1-py38hacc1119_0.conda",
        "sphinxcontrib-programoutput-0.11-py36ha61c614_0.tar.bz2",
        "sphinxcontrib-programoutput-0.11-py37h79f54e7_0.tar.bz2",
        "sphinxcontrib-programoutput-0.14-py36_0.tar.bz2",
        "sphinxcontrib-programoutput-0.14-py37_0.tar.bz2",
        "sphinxcontrib-programoutput-0.15-py36_0.conda",
        "sphinxcontrib-programoutput-0.15-py37_0.conda",
        "sphinxcontrib-programoutput-0.16-py36_0.conda",
        "sphinxcontrib-programoutput-0.16-py36_1.conda",
        "sphinxcontrib-programoutput-0.16-py37_0.conda",
        "sphinxcontrib-programoutput-0.16-py37_1.conda",
        "sphinxcontrib-programoutput-0.16-py38_1.conda",
        "torchvision-0.8.1-py36_cpu.tar.bz2",
        "torchvision-0.8.1-py37_cpu.tar.bz2",
        "torchvision-0.8.1-py38_cpu.tar.bz2",
        "vlfeat-0.9.20-0.tar.bz2",
        "vlfeat-0.9.20-1.tar.bz2",
        "vlfeat-0.9.20-2.tar.bz2",
        "vlfeat-0.9.20-3.tar.bz2",
        "vlfeat-0.9.21-h1de35cc_1.conda",
        "vlfeat-0.9.21-h2d5a098_0.tar.bz2",
        "x264-20131217-0.tar.bz2",
        "x264-20131217-1.tar.bz2",
        "x264-20131217-2.tar.bz2",
        "x264-20131217-3.tar.bz2",
        "zc.buildout-2.10.0-py27_0.tar.bz2",
        "zc.buildout-2.10.0-py35_0.tar.bz2",
        "zc.buildout-2.10.0-py36_0.tar.bz2",
        "zc.buildout-2.11.2-py27_0.tar.bz2",
        "zc.buildout-2.11.2-py36_0.tar.bz2",
        "zc.buildout-2.12.2-py36_0.tar.bz2",
        "zc.buildout-2.12.2-py37_0.tar.bz2",
        "zc.buildout-2.13.2-py36_0.tar.bz2",
        "zc.buildout-2.13.2-py36_1.conda",
        "zc.buildout-2.13.2-py37_0.tar.bz2",
        "zc.buildout-2.13.2-py37_1.conda",
        "zc.buildout-2.13.3-py36_1.conda",
        "zc.buildout-2.13.3-py36_2.conda",
        "zc.buildout-2.13.3-py37_1.conda",
        "zc.buildout-2.13.3-py37_2.conda",
        "zc.buildout-2.13.3-py38_2.conda",
        "zc.buildout-2.9.2-py27_0.tar.bz2",
        "zc.buildout-2.9.2-py35_0.tar.bz2",
        "zc.buildout-2.9.2-py36_0.tar.bz2",
        "zc.buildout-2.9.5-py27_0.tar.bz2",
        "zc.buildout-2.9.5-py35_0.tar.bz2",
        "zc.buildout-2.9.5-py36_0.tar.bz2",
        "zc.recipe.egg-2.0.3-py27_0.tar.bz2",
        "zc.recipe.egg-2.0.3-py35_0.tar.bz2",
        "zc.recipe.egg-2.0.3-py36_0.tar.bz2",
        "zc.recipe.egg-2.0.4-py27_0.tar.bz2",
        "zc.recipe.egg-2.0.4-py35_0.tar.bz2",
        "zc.recipe.egg-2.0.4-py36_0.tar.bz2",
        "zc.recipe.egg-2.0.5-py27_0.tar.bz2",
        "zc.recipe.egg-2.0.5-py35_0.tar.bz2",
        "zc.recipe.egg-2.0.5-py36_0.tar.bz2",
        "zc.recipe.egg-2.0.7-py36h6217847_0.tar.bz2",
        "zc.recipe.egg-2.0.7-py36h8c28fb6_0.conda",
        "zc.recipe.egg-2.0.7-py36he61d863_1.conda",
        "zc.recipe.egg-2.0.7-py37h8c28fb6_0.conda",
        "zc.recipe.egg-2.0.7-py37h8c28fb6_0.tar.bz2",
        "zc.recipe.egg-2.0.7-py37he61d863_1.conda",
        "zc.recipe.egg-2.0.7-py38he61d863_1.conda",
    },
    "linux-64": {
        "aniso8601-3.0.0-py27haa2c913_0.tar.bz2",
        "aniso8601-3.0.0-py36haa2c913_0.tar.bz2",
        "aniso8601-4.0.1-py36h6217847_0.tar.bz2",
        "aniso8601-4.0.1-py36h8c28fb6_0.conda",
        "aniso8601-4.0.1-py37h8c28fb6_0.conda",
        "aniso8601-4.0.1-py37h8c28fb6_0.tar.bz2",
        "aspy.yaml-1.1.1-py36h6217847_0.tar.bz2",
        "aspy.yaml-1.1.1-py36h8c28fb6_0.conda",
        "aspy.yaml-1.1.1-py37h8c28fb6_0.conda",
        "aspy.yaml-1.1.1-py37h8c28fb6_0.tar.bz2",
        "cfgv-1.4.0-py36_0.conda",
        "cfgv-1.4.0-py36h493f39d_0.tar.bz2",
        "cfgv-1.4.0-py37_0.conda",
        "cfgv-1.4.0-py37h81a5984_0.tar.bz2",
        "colorlog-3.1.4-py27haf68d3b_0.tar.bz2",
        "colorlog-3.1.4-py36haf68d3b_0.tar.bz2",
        "cyvlfeat-0.4.5-np111py27_0.tar.bz2",
        "cyvlfeat-0.4.5-np111py27_1.tar.bz2",
        "cyvlfeat-0.4.5-np111py27_2.tar.bz2",
        "cyvlfeat-0.4.5-np111py34_0.tar.bz2",
        "cyvlfeat-0.4.5-np111py34_1.tar.bz2",
        "cyvlfeat-0.4.5-np111py34_2.tar.bz2",
        "cyvlfeat-0.4.5-np111py35_0.tar.bz2",
        "cyvlfeat-0.4.5-np111py35_1.tar.bz2",
        "cyvlfeat-0.4.5-np111py35_2.tar.bz2",
        "cyvlfeat-0.4.5-np111py36_2.tar.bz2",
        "cyvlfeat-0.4.5-np112py27_2.tar.bz2",
        "cyvlfeat-0.4.5-np112py35_2.tar.bz2",
        "cyvlfeat-0.4.5-np112py36_2.tar.bz2",
        "cyvlfeat-0.4.5-np113py27_2.tar.bz2",
        "cyvlfeat-0.4.5-np113py35_2.tar.bz2",
        "cyvlfeat-0.4.5-np113py36_2.tar.bz2",
        "cyvlfeat-0.4.6-np112py27_2.tar.bz2",
        "cyvlfeat-0.4.6-np112py35_2.tar.bz2",
        "cyvlfeat-0.4.6-np112py36_2.tar.bz2",
        "cyvlfeat-0.4.6-np113py27_2.tar.bz2",
        "cyvlfeat-0.4.6-np113py35_2.tar.bz2",
        "cyvlfeat-0.4.6-np113py36_2.tar.bz2",
        "cyvlfeat-0.4.6-py27hc2e7fa5_3.tar.bz2",
        "cyvlfeat-0.4.6-py27hd9be493_4.tar.bz2",
        "cyvlfeat-0.4.6-py36hc2e7fa5_3.tar.bz2",
        "cyvlfeat-0.4.6-py36hd9be493_4.tar.bz2",
        "cyvlfeat-0.4.6-py37h894a32a_4.tar.bz2",
        "cyvlfeat-0.5.1-py36h7ca735f_0.tar.bz2",
        "cyvlfeat-0.5.1-py36h7ca735f_1.conda",
        "cyvlfeat-0.5.1-py37h7ca735f_0.tar.bz2",
        "cyvlfeat-0.5.1-py37h7ca735f_1.conda",
        "dataclasses-0.8-py36_0.conda",
        "dataclasses-0.8-py37_0.conda",
        "dataclasses-0.8-py37h0146b56_1.conda",
        "dataclasses-0.8-py38h0146b56_0.conda",
        "dataclasses-0.8-py38h0146b56_1.conda",
        "ddt-1.2.1-py36h8c28fb6_1.conda",
        "ddt-1.2.1-py36he61d863_2.conda",
        "ddt-1.2.1-py37h8c28fb6_1.conda",
        "ddt-1.2.1-py37he61d863_2.conda",
        "ddt-1.2.1-py38he61d863_2.conda",
        "dlib-19.0-np111py27_0.tar.bz2",
        "dlib-19.0-np111py27_1.tar.bz2",
        "dlib-19.0-np111py35_0.tar.bz2",
        "dlib-19.0-np111py35_1.tar.bz2",
        "dlib-19.0-np111py36_0.tar.bz2",
        "dlib-19.0-np111py36_1.tar.bz2",
        "dlib-19.0-np112py27_0.tar.bz2",
        "dlib-19.0-np112py27_1.tar.bz2",
        "dlib-19.0-np112py35_0.tar.bz2",
        "dlib-19.0-np112py35_1.tar.bz2",
        "dlib-19.0-np112py36_0.tar.bz2",
        "dlib-19.0-np112py36_1.tar.bz2",
        "dlib-19.0-np113py27_1.tar.bz2",
        "dlib-19.0-np113py35_1.tar.bz2",
        "dlib-19.0-np113py36_1.tar.bz2",
        "dlib-19.7-py27h058b003_0.tar.bz2",
        "dlib-19.7-py27hdafcfc0_1.tar.bz2",
        "dlib-19.7-py36h058b003_0.tar.bz2",
        "dlib-19.7-py36h17ab140_2.conda",
        "dlib-19.7-py36h470cf47_2.tar.bz2",
        "dlib-19.7-py36h49bf850_3.conda",
        "dlib-19.7-py36hdafcfc0_1.tar.bz2",
        "dlib-19.7-py37h17ab140_2.conda",
        "dlib-19.7-py37h49bf850_3.conda",
        "dlib-19.7-py37hf1bcec1_2.tar.bz2",
        "docs-0.2.1b0-h7d4bfc7_0.conda",
        "docs-0.3.1-h7d4bfc7_0.conda",
        "docs-2018.02.21b0-h54c6e9d_3.conda",
        "docs-2018.02.21b0-h755dffa_1.tar.bz2",
        "docs-2018.02.21b0-h755dffa_2.tar.bz2",
        "docs-2018.02.21b0-h794fcec_4.conda",
        "docs-2018.02.21b0-habdb222_0.tar.bz2",
        "ffmpeg-2.8.10-0.tar.bz2",
        "ffmpeg-2.8.10-1.tar.bz2",
        "ffmpeg-2.8.10-2.tar.bz2",
        "ffmpeg-2.8.10-3.tar.bz2",
        "ffmpeg-2.8.10-4.tar.bz2",
        "ffmpeg-3.4-hdec9c9a_1.tar.bz2",
        "ffmpeg-4.0-hadceb68_1.tar.bz2",
        "ffmpeg-4.0-hdb0e523_0.tar.bz2",
        "ffmpeg-4.0-hf6c5f75_2.conda",
        "flask-restful-0.3.6-py27haa2c913_0.tar.bz2",
        "flask-restful-0.3.6-py36h8c28fb6_0.conda",
        "flask-restful-0.3.6-py36haa2c913_0.tar.bz2",
        "flask-restful-0.3.6-py37h4c55d99_0.tar.bz2",
        "flask-restful-0.3.6-py37h8c28fb6_0.conda",
        "identify-1.1.8-py36_0.conda",
        "identify-1.1.8-py36_0.tar.bz2",
        "identify-1.1.8-py37_0.conda",
        "identify-1.1.8-py37_0.tar.bz2",
        "importlib_resources-1.0.2-py36_0.conda",
        "importlib_resources-1.0.2-py36_0.tar.bz2",
        "importlib_resources-1.0.2-py37_0.conda",
        "importlib_resources-1.0.2-py37_0.tar.bz2",
        "ipdb-0.10.2-py27_0.tar.bz2",
        "ipdb-0.10.2-py34_0.tar.bz2",
        "ipdb-0.10.2-py35_0.tar.bz2",
        "ipdb-0.10.2-py36_0.tar.bz2",
        "ipdb-0.10.3-py27h6a1c9b6_0.tar.bz2",
        "ipdb-0.10.3-py36h6a1c9b6_0.tar.bz2",
        "ipdb-0.11-py36h30e596e_0.tar.bz2",
        "ipdb-0.11-py37hbf0455a_0.tar.bz2",
        "ipdb-0.12.2-py36h8c28fb6_0.tar.bz2",
        "ipdb-0.12.2-py37h8c28fb6_0.tar.bz2",
        "ipdb-0.12.3-py36h8c28fb6_0.conda",
        "ipdb-0.12.3-py37h8c28fb6_0.conda",
        "ipdb-0.13.2-py36h8c28fb6_0.conda",
        "ipdb-0.13.2-py37h8c28fb6_0.conda",
        "ipdb-0.13.3-py36h8c28fb6_0.conda",
        "ipdb-0.13.3-py37h8c28fb6_0.conda",
        "kaldi-1!5.5.164-h13f0c7c_0.conda",
        "kaldi-1!5.5.164-h13f0c7c_0.tar.bz2",
        "kaldi-1!5.5.164-h93a79c4_1.conda",
        "kaldi-2017.03.13-h6bb2d05_3.tar.bz2",
        "kaldi-5.5.164-h93a79c4_0.conda",
        "kaldi-r7271.1a4dbf6-0.tar.bz2",
        "kaldi-r7271.1a4dbf6-h6bb2d05_2.tar.bz2",
        "libblitz-0.10-0.tar.bz2",
        "libblitz-0.10-1.tar.bz2",
        "libblitz-0.10-2.tar.bz2",
        "libblitz-0.10-3.tar.bz2",
        "libblitz-1.0.1-h5c30f38_0.tar.bz2",
        "libblitz-1.0.1-hf484d3e_1.conda",
        "libmatio-1.5.11-h35b33d1_1.tar.bz2",
        "libmatio-1.5.11-ha500df2_0.tar.bz2",
        "libmatio-1.5.15-h0f1def3_0.tar.bz2",
        "libmatio-1.5.15-h35b33d1_0.tar.bz2",
        "libmatio-1.5.15-hfbefa3d_0.tar.bz2",
        "libmatio-1.5.17-h0f1def3_0.conda",
        "libmatio-1.5.17-h35b33d1_0.conda",
        "libmatio-1.5.6-0.tar.bz2",
        "libmatio-1.5.6-1.tar.bz2",
        "libmatio-1.5.6-2.tar.bz2",
        "libmatio-1.5.6-3.tar.bz2",
        "libopencv-4.5.0-py37_0.conda",
        "libopencv-4.5.0-py37_1.conda",
        "libopencv-4.5.0-py38_1.conda",
        "libsvm-3.21-0.tar.bz2",
        "libsvm-3.21-1.tar.bz2",
        "libsvm-3.21-2.tar.bz2",
        "libsvm-3.21-3.tar.bz2",
        "libsvm-3.22-h4f306d9_0.tar.bz2",
        "libsvm-3.24-hf484d3e_0.conda",
        "madmom-0.15.1-py27_0.tar.bz2",
        "madmom-0.15.1-py27h2137316_2.tar.bz2",
        "madmom-0.15.1-py27hf55e54f_1.tar.bz2",
        "madmom-0.15.1-py35_0.tar.bz2",
        "madmom-0.15.1-py36_0.tar.bz2",
        "madmom-0.15.1-py36h2137316_2.tar.bz2",
        "madmom-0.15.1-py36hf55e54f_1.tar.bz2",
        "madmom-0.16.1-py36h573f956_0.tar.bz2",
        "madmom-0.16.1-py37h573f956_0.tar.bz2",
        "menpo-0.7.7-np111py27_0.tar.bz2",
        "menpo-0.7.7-np111py27_1.tar.bz2",
        "menpo-0.7.7-np111py27_2.tar.bz2",
        "menpo-0.7.7-np111py27_3.tar.bz2",
        "menpo-0.7.7-np111py27_4.tar.bz2",
        "menpo-0.7.7-np111py34_0.tar.bz2",
        "menpo-0.7.7-np111py34_1.tar.bz2",
        "menpo-0.7.7-np111py34_2.tar.bz2",
        "menpo-0.7.7-np111py34_3.tar.bz2",
        "menpo-0.7.7-np111py34_4.tar.bz2",
        "menpo-0.7.7-np111py35_0.tar.bz2",
        "menpo-0.7.7-np111py35_1.tar.bz2",
        "menpo-0.7.7-np111py35_2.tar.bz2",
        "menpo-0.7.7-np111py35_3.tar.bz2",
        "menpo-0.7.7-np111py35_4.tar.bz2",
        "menpo-0.7.7-np111py36_4.tar.bz2",
        "menpo-0.7.7-np112py27_4.tar.bz2",
        "menpo-0.7.7-np112py35_4.tar.bz2",
        "menpo-0.7.7-np112py36_4.tar.bz2",
        "menpo-0.7.7-np113py27_4.tar.bz2",
        "menpo-0.7.7-np113py35_4.tar.bz2",
        "menpo-0.7.7-np113py36_4.tar.bz2",
        "menpo-0.8.1-np112py27_0.tar.bz2",
        "menpo-0.8.1-np112py35_0.tar.bz2",
        "menpo-0.8.1-np112py36_0.tar.bz2",
        "menpo-0.8.1-np113py27_0.tar.bz2",
        "menpo-0.8.1-np113py35_0.tar.bz2",
        "menpo-0.8.1-np113py36_0.tar.bz2",
        "menpo-0.8.1-py27h71df38f_4.tar.bz2",
        "menpo-0.8.1-py27h71df38f_5.tar.bz2",
        "menpo-0.8.1-py27haa059fb_3.tar.bz2",
        "menpo-0.8.1-py27hfd4ac0d_6.tar.bz2",
        "menpo-0.8.1-py36h71df38f_4.tar.bz2",
        "menpo-0.8.1-py36h71df38f_5.tar.bz2",
        "menpo-0.8.1-py36haa059fb_3.tar.bz2",
        "menpo-0.8.1-py36hab74019_8.tar.bz2",
        "menpo-0.8.1-py36hf56cdcd_7.tar.bz2",
        "menpo-0.8.1-py36hfd4ac0d_6.tar.bz2",
        "menpo-0.8.1-py37hab74019_8.tar.bz2",
        "menpo-0.9.1-py36h49f2cb8_0.tar.bz2",
        "menpo-0.9.1-py36h49f2cb8_1.tar.bz2",
        "menpo-0.9.1-py37h49f2cb8_0.tar.bz2",
        "menpo-0.9.1-py37h49f2cb8_1.tar.bz2",
        "menpo-0.9.2-py36h49f2cb8_0.conda",
        "menpo-0.9.2-py37h49f2cb8_0.conda",
        "menpofit-0.4.1-py27_0.tar.bz2",
        "menpofit-0.4.1-py27_1.tar.bz2",
        "menpofit-0.4.1-py34_0.tar.bz2",
        "menpofit-0.4.1-py34_1.tar.bz2",
        "menpofit-0.4.1-py35_0.tar.bz2",
        "menpofit-0.4.1-py35_1.tar.bz2",
        "menpofit-0.4.1-py36_1.tar.bz2",
        "menpofit-0.5.0-py27_0.tar.bz2",
        "menpofit-0.5.0-py35_0.tar.bz2",
        "menpofit-0.5.0-py36_0.tar.bz2",
        "menpofit-0.5.0-py37hbf0455a_0.tar.bz2",
        "menpofit-0.6.1-py36h8c28fb6_0.tar.bz2",
        "menpofit-0.6.1-py36h8c28fb6_1.tar.bz2",
        "menpofit-0.6.1-py36h8c28fb6_2.conda",
        "menpofit-0.6.1-py37h8c28fb6_0.tar.bz2",
        "menpofit-0.6.1-py37h8c28fb6_1.tar.bz2",
        "menpofit-0.6.1-py37h8c28fb6_2.conda",
        "menpowidgets-0.2.1p3-py27_0.tar.bz2",
        "menpowidgets-0.2.1p3-py34_0.tar.bz2",
        "menpowidgets-0.2.1p3-py35_0.tar.bz2",
        "menpowidgets-0.2.1p3-py36_0.tar.bz2",
        "menpowidgets-0.3.0p6-py27_0.tar.bz2",
        "menpowidgets-0.3.0p6-py35_0.tar.bz2",
        "menpowidgets-0.3.0p6-py36_0.tar.bz2",
        "mido-1.2.9-py36h8c28fb6_0.tar.bz2",
        "mido-1.2.9-py37h8c28fb6_0.tar.bz2",
        "mne-0.13.1-np111py27_0.tar.bz2",
        "mne-0.13.1-np111py34_0.tar.bz2",
        "mne-0.13.1-np111py35_0.tar.bz2",
        "mne-0.13.1-np111py36_0.tar.bz2",
        "mne-0.13.1-np112py27_0.tar.bz2",
        "mne-0.13.1-np112py35_0.tar.bz2",
        "mne-0.13.1-np112py36_0.tar.bz2",
        "mne-0.13.1-np113py27_0.tar.bz2",
        "mne-0.13.1-np113py35_0.tar.bz2",
        "mne-0.13.1-np113py36_0.tar.bz2",
        "mne-0.14.1-np112py27_0.tar.bz2",
        "mne-0.14.1-np112py35_0.tar.bz2",
        "mne-0.14.1-np112py36_0.tar.bz2",
        "mne-0.14.1-np113py27_0.tar.bz2",
        "mne-0.14.1-np113py35_0.tar.bz2",
        "mne-0.14.1-np113py36_0.tar.bz2",
        "mne-0.15.2-py27h249d46d_0.tar.bz2",
        "mne-0.15.2-py36h249d46d_0.tar.bz2",
        "mne-0.15.2-py37he5954f6_0.tar.bz2",
        "mne-0.18.1-py36h8c28fb6_0.conda",
        "mne-0.18.1-py36h8c28fb6_0.tar.bz2",
        "mne-0.18.1-py37h8c28fb6_0.conda",
        "mne-0.18.1-py37h8c28fb6_0.tar.bz2",
        "mr.developer-1.38-py27_0.tar.bz2",
        "mr.developer-1.38-py35_0.tar.bz2",
        "mr.developer-1.38-py36_0.tar.bz2",
        "mr.developer-1.38-py37h8c28fb6_0.tar.bz2",
        "mr.developer-2.0.0-py36h8c28fb6_0.conda",
        "mr.developer-2.0.0-py36h8c28fb6_0.tar.bz2",
        "mr.developer-2.0.0-py37h8c28fb6_0.conda",
        "mr.developer-2.0.0-py37h8c28fb6_0.tar.bz2",
        "nodeenv-1.3.3-py36h6217847_0.tar.bz2",
        "nodeenv-1.3.3-py36h8c28fb6_0.conda",
        "nodeenv-1.3.3-py37h8c28fb6_0.conda",
        "nodeenv-1.3.3-py37h8c28fb6_0.tar.bz2",
        "opencv-4.5.0-py37_0.conda",
        "opencv-4.5.0-py37_1.conda",
        "opencv-4.5.0-py38_1.conda",
        "openfst-1.6.1-0.tar.bz2",
        "openfst-1.6.1-hf484d3e_2.tar.bz2",
        "openfst-1.6.7-hf484d3e_0.conda",
        "openfst-1.6.7-hf484d3e_0.tar.bz2",
        "openh264-1.7.0-hc521636_1.tar.bz2",
        "openh264-1.7.0-hf6c5f75_2.conda",
        "oset-0.1.3-py27haf68d3b_0.tar.bz2",
        "oset-0.1.3-py36haf68d3b_0.tar.bz2",
        "oset-0.1.3-py37h3fd5425_0.tar.bz2",
        "pkgtools-0.7.3-py27_0.tar.bz2",
        "pkgtools-0.7.3-py34_0.tar.bz2",
        "pkgtools-0.7.3-py35_0.tar.bz2",
        "pkgtools-0.7.3-py36_0.tar.bz2",
        "pre_commit-1.13.0-py36_0.tar.bz2",
        "pre_commit-1.13.0-py36_1.conda",
        "pre_commit-1.13.0-py36_1.tar.bz2",
        "pre_commit-1.13.0-py37_0.tar.bz2",
        "pre_commit-1.13.0-py37_1.conda",
        "pre_commit-1.13.0-py37_1.tar.bz2",
        "py-opencv-4.5.0-py37h73dd053_0.conda",
        "py-opencv-4.5.0-py37h73dd053_1.conda",
        "py-opencv-4.5.0-py38hdb77d25_1.conda",
        "pyedflib-0.1.11-py27h1d7afee_1.tar.bz2",
        "pyedflib-0.1.11-py27h5a3ec8c_0.tar.bz2",
        "pyedflib-0.1.11-py36h1d7afee_1.tar.bz2",
        "pyedflib-0.1.11-py36h5a3ec8c_0.tar.bz2",
        "pyedflib-0.1.11-py37hfdfbdf1_1.tar.bz2",
        "pyedflib-0.1.14-py36h0c21175_0.conda",
        "pyedflib-0.1.14-py36h0c21175_0.tar.bz2",
        "pyedflib-0.1.14-py37h0c21175_0.conda",
        "pyedflib-0.1.14-py37h0c21175_0.tar.bz2",
        "pyedflib-0.1.6-np111py27_0.tar.bz2",
        "pyedflib-0.1.6-np111py27_1.tar.bz2",
        "pyedflib-0.1.6-np111py27_2.tar.bz2",
        "pyedflib-0.1.6-np111py34_0.tar.bz2",
        "pyedflib-0.1.6-np111py34_1.tar.bz2",
        "pyedflib-0.1.6-np111py34_2.tar.bz2",
        "pyedflib-0.1.6-np111py35_0.tar.bz2",
        "pyedflib-0.1.6-np111py35_1.tar.bz2",
        "pyedflib-0.1.6-np111py35_2.tar.bz2",
        "pyedflib-0.1.6-np111py36_2.tar.bz2",
        "pyedflib-0.1.6-np112py27_2.tar.bz2",
        "pyedflib-0.1.6-np112py35_2.tar.bz2",
        "pyedflib-0.1.6-np112py36_2.tar.bz2",
        "pyedflib-0.1.6-np113py27_2.tar.bz2",
        "pyedflib-0.1.6-np113py35_2.tar.bz2",
        "pyedflib-0.1.6-np113py36_2.tar.bz2",
        "pyedflib-0.1.9-np112py27_0.tar.bz2",
        "pyedflib-0.1.9-np112py35_0.tar.bz2",
        "pyedflib-0.1.9-np112py36_0.tar.bz2",
        "pyedflib-0.1.9-np113py27_0.tar.bz2",
        "pyedflib-0.1.9-np113py35_0.tar.bz2",
        "pyedflib-0.1.9-np113py36_0.tar.bz2",
        "pytables-3.6.1-py37h1ac3923_0.conda",
        "pytables-3.6.1-py38h1ac3923_0.conda",
        "pytest-qt-3.2.2-py36h6217847_0.tar.bz2",
        "pytest-qt-3.2.2-py36h8c28fb6_0.conda",
        "pytest-qt-3.2.2-py36he61d863_1.conda",
        "pytest-qt-3.2.2-py37h8c28fb6_0.conda",
        "pytest-qt-3.2.2-py37h8c28fb6_0.tar.bz2",
        "pytest-qt-3.2.2-py37he61d863_1.conda",
        "pytest-qt-3.2.2-py38he61d863_1.conda",
        "python-gitlab-1.10.0-py36_0.tar.bz2",
        "python-gitlab-1.10.0-py37_0.tar.bz2",
        "python-gitlab-1.12.1-py36_0.conda",
        "python-gitlab-1.12.1-py37_0.conda",
        "python-gitlab-1.15.0-py36_0.conda",
        "python-gitlab-1.15.0-py37_0.conda",
        "python-gitlab-1.7.0-py36_0.tar.bz2",
        "python-gitlab-1.7.0-py36_1.tar.bz2",
        "python-gitlab-1.7.0-py37_1.tar.bz2",
        "python-gitlab-2.1.2-py36_0.conda",
        "python-gitlab-2.1.2-py37_0.conda",
        "python-gitlab-2.2.0-py36_0.conda",
        "python-gitlab-2.2.0-py37_0.conda",
        "python-gitlab-2.2.0-py38_0.conda",
        "python-gitlab-2.2.0-py39_0.conda",
        "python-gitlab-2.6.0-py37_0.conda",
        "python-gitlab-2.6.0-py38_0.conda",
        "python-gitlab-2.6.0-py39_0.conda",
        "pytorch-1.7.0-py3.7_cuda11.0.221_cudnn8.0.3_0.tar.bz2",
        "pytorch-1.7.0-py3.8_cuda11.0.221_cudnn8.0.3_0.tar.bz2",
        "schema-0.6.5-py27_0.tar.bz2",
        "schema-0.6.5-py34_0.tar.bz2",
        "schema-0.6.5-py35_0.tar.bz2",
        "schema-0.6.5-py36_0.tar.bz2",
        "schema-0.6.7-py27h91e9215_0.tar.bz2",
        "schema-0.6.7-py36h91e9215_0.tar.bz2",
        "schema-0.6.8-py36h426cc52_0.tar.bz2",
        "schema-0.6.8-py37he4c3ae9_0.tar.bz2",
        "schema-0.7.0-py36h8c28fb6_0.tar.bz2",
        "schema-0.7.0-py36h8c28fb6_1.tar.bz2",
        "schema-0.7.0-py37h8c28fb6_0.tar.bz2",
        "schema-0.7.0-py37h8c28fb6_1.tar.bz2",
        "schema-0.7.1-py36h8c28fb6_0.conda",
        "schema-0.7.1-py37h8c28fb6_0.conda",
        "schema-0.7.2-py36he61d863_0.conda",
        "schema-0.7.2-py37he61d863_0.conda",
        "schema-0.7.2-py38he61d863_0.conda",
        "schema-0.7.4-py37h6674461_0.conda",
        "schema-0.7.4-py38h6674461_0.conda",
        "sox-14.4.2-0.tar.bz2",
        "sox-14.4.2-1.tar.bz2",
        "sox-14.4.2-2.tar.bz2",
        "sox-14.4.2-3.tar.bz2",
        "sox-14.4.2-4.tar.bz2",
        "sox-14.4.2-5.tar.bz2",
        "sox-14.4.2-hc3418cb_8.conda",
        "sox-14.4.2-hf1d46b3_7.tar.bz2",
        "speex-1.2.0-0.tar.bz2",
        "speex-1.2.0-hb7c1d73_1.conda",
        "speex-1.2.0-hb7c1d73_1.tar.bz2",
        "speexdsp-1.2rc3-h5bbff6d_0.tar.bz2",
        "speexdsp-1.2rc3-hf484d3e_0.conda",
        "sphinxcontrib-httpdomain-1.7.0-py36_0.tar.bz2",
        "sphinxcontrib-httpdomain-1.7.0-py36_1.conda",
        "sphinxcontrib-httpdomain-1.7.0-py36_2.conda",
        "sphinxcontrib-httpdomain-1.7.0-py37_0.tar.bz2",
        "sphinxcontrib-httpdomain-1.7.0-py37_1.conda",
        "sphinxcontrib-httpdomain-1.7.0-py37_2.conda",
        "sphinxcontrib-httpdomain-1.7.0-py38_2.conda",
        "sphinxcontrib-mermaid-0.6.1-py37h0146b56_0.conda",
        "sphinxcontrib-mermaid-0.6.1-py38h0146b56_0.conda",
        "sphinxcontrib-programoutput-0.11-py36ha61c614_0.tar.bz2",
        "sphinxcontrib-programoutput-0.11-py37h79f54e7_0.tar.bz2",
        "sphinxcontrib-programoutput-0.14-py36_0.tar.bz2",
        "sphinxcontrib-programoutput-0.14-py37_0.tar.bz2",
        "sphinxcontrib-programoutput-0.15-py36_0.conda",
        "sphinxcontrib-programoutput-0.15-py37_0.conda",
        "sphinxcontrib-programoutput-0.16-py36_0.conda",
        "sphinxcontrib-programoutput-0.16-py36_1.conda",
        "sphinxcontrib-programoutput-0.16-py37_0.conda",
        "sphinxcontrib-programoutput-0.16-py37_1.conda",
        "sphinxcontrib-programoutput-0.16-py38_1.conda",
        "torchvision-0.8.1-py37_cu110.tar.bz2",
        "torchvision-0.8.1-py38_cu110.tar.bz2",
        "vlfeat-0.9.20-0.tar.bz2",
        "vlfeat-0.9.20-1.tar.bz2",
        "vlfeat-0.9.20-2.tar.bz2",
        "vlfeat-0.9.20-3.tar.bz2",
        "vlfeat-0.9.21-h14c3975_1.conda",
        "vlfeat-0.9.21-h18fa195_0.tar.bz2",
        "x264-20131217-0.tar.bz2",
        "x264-20131217-1.tar.bz2",
        "x264-20131217-2.tar.bz2",
        "x264-20131217-3.tar.bz2",
        "zc.buildout-2.10.0-py27_0.tar.bz2",
        "zc.buildout-2.10.0-py35_0.tar.bz2",
        "zc.buildout-2.10.0-py36_0.tar.bz2",
        "zc.buildout-2.11.2-py27_0.tar.bz2",
        "zc.buildout-2.11.2-py36_0.tar.bz2",
        "zc.buildout-2.12.2-py36_0.tar.bz2",
        "zc.buildout-2.12.2-py37_0.tar.bz2",
        "zc.buildout-2.13.2-py36_0.tar.bz2",
        "zc.buildout-2.13.2-py36_1.conda",
        "zc.buildout-2.13.2-py37_0.tar.bz2",
        "zc.buildout-2.13.2-py37_1.conda",
        "zc.buildout-2.13.3-py36_1.conda",
        "zc.buildout-2.13.3-py36_2.conda",
        "zc.buildout-2.13.3-py37_1.conda",
        "zc.buildout-2.13.3-py37_2.conda",
        "zc.buildout-2.13.3-py38_2.conda",
        "zc.buildout-2.9.2-py27_0.tar.bz2",
        "zc.buildout-2.9.2-py35_0.tar.bz2",
        "zc.buildout-2.9.2-py36_0.tar.bz2",
        "zc.buildout-2.9.5-py27_0.tar.bz2",
        "zc.buildout-2.9.5-py35_0.tar.bz2",
        "zc.buildout-2.9.5-py36_0.tar.bz2",
        "zc.recipe.egg-2.0.3-py27_0.tar.bz2",
        "zc.recipe.egg-2.0.3-py35_0.tar.bz2",
        "zc.recipe.egg-2.0.3-py36_0.tar.bz2",
        "zc.recipe.egg-2.0.4-py27_0.tar.bz2",
        "zc.recipe.egg-2.0.4-py35_0.tar.bz2",
        "zc.recipe.egg-2.0.4-py36_0.tar.bz2",
        "zc.recipe.egg-2.0.5-py27_0.tar.bz2",
        "zc.recipe.egg-2.0.5-py35_0.tar.bz2",
        "zc.recipe.egg-2.0.5-py36_0.tar.bz2",
        "zc.recipe.egg-2.0.7-py36h6217847_0.tar.bz2",
        "zc.recipe.egg-2.0.7-py36h8c28fb6_0.conda",
        "zc.recipe.egg-2.0.7-py36he61d863_1.conda",
        "zc.recipe.egg-2.0.7-py37h8c28fb6_0.conda",
        "zc.recipe.egg-2.0.7-py37h8c28fb6_0.tar.bz2",
        "zc.recipe.egg-2.0.7-py37he61d863_1.conda",
        "zc.recipe.egg-2.0.7-py38he61d863_1.conda",
    },
    "noarch": {
        "cpuonly-1.0-0.tar.bz2",
        "ddt-1.2.1-pyh6b12c4a_0.tar.bz2",
        "ddt-1.2.1-pyh8c28fb6_0.conda",
        "docformatter-1.3-py_0.conda",
        "docformatter-1.3-pyhd6b4573_0.tar.bz2",
        "docformatter-1.3.1-py_0.conda",
        "docformatter-1.4-py_0.conda",
        "ratelimiter-1.2.0.post0-pyh8c28fb6_0.tar.bz2",
        "untokenize-0.1.1-py_0.conda",
        "untokenize-0.1.1-py_1.conda",
        "untokenize-0.1.1-pyhd6b4573_0.tar.bz2",
    },
}

OPERATORS = ["==", ">=", "<=", ">", "<", "!="]


def _add_removals(instructions, currvals):
    instructions["remove"].extend(tuple(set(currvals)))


def _gen_patch_instructions(index, new_index, packages_key):
    instructions = {
        "patch_instructions_version": 1,
        packages_key: defaultdict(dict),
        "revoke": [],
        "remove": [],
    }

    # diff all items in the index and put any differences in the instructions
    for fn in index:
        assert fn in new_index

        # replace any old keys
        for key in index[fn]:
            assert key in new_index[fn], (key, index[fn], new_index[fn])
            if index[fn][key] != new_index[fn][key]:
                instructions[packages_key][fn][key] = new_index[fn][key]

        # add any new keys
        for key in new_index[fn]:
            if key not in index[fn]:
                instructions[packages_key][fn][key] = new_index[fn][key]

    return instructions


def _gen_new_index(repodata, packages_key):
    """Make any changes to the index by adjusting the values directly.

    This function returns the new index with the adjustments.
    Finally, the new and old indices are then diff'ed to produce the repo
    data patches.
    """
    index = copy.deepcopy(repodata[packages_key])

    for fn, record in index.items():
        record_name = record["name"]

        # bob.bio.base <=4.1.0 does not work with numpy >=1.18
        if record_name == "bob.bio.base":
            if parse_version(record["version"]) <= parse_version("4.1.0"):
                record["depends"].append("numpy <1.18")

        # pytorch 1.7.0 0 packages need this extra dependencies to run
        if record_name == "pytorch":
            if record["version"] == "1.7.0" and record["build_number"] == 0:
                record["depends"].append("dataclasses >=0.8")
                record["depends"].append("future >=0.18.2")

    return index


def gen_new_index_and_patch_instructions(repodata):
    instructions = {}
    for i, packages_key in enumerate(["packages", "packages.conda"]):
        new_index = _gen_new_index(repodata, packages_key)
        inst = _gen_patch_instructions(
            repodata[packages_key], new_index, packages_key
        )
        _add_removals(inst, REMOVALS[repodata["info"]["subdir"]])
        if i == 0:
            instructions.update(inst)
        else:
            instructions[packages_key] = inst[packages_key]
            instructions["revoke"] = list(
                set(instructions["revoke"]) | set(inst["revoke"])
            )
            instructions["remove"] = list(
                set(instructions["remove"]) | set(inst["remove"])
            )

    return instructions


def main():
    # Step 1. Collect initial repodata for all subdirs.
    repodatas = {}
    subdirs = SUBDIRS
    for subdir in tqdm.tqdm(subdirs, desc="Downloading repodata"):
        repodata_url = "/".join(
            (BASE_URL, subdir, "repodata_from_packages.json")
        )
        response = requests.get(repodata_url)
        response.raise_for_status()
        repodatas[subdir] = response.json()

    # Step 2. Create all patch instructions.
    prefix_dir = os.getenv("PREFIX", "tmp")
    for subdir in subdirs:
        prefix_subdir = join(prefix_dir, subdir)
        if not isdir(prefix_subdir):
            os.makedirs(prefix_subdir)

        # Step 2a. Generate a new index.
        # Step 2b. Generate the instructions by diff'ing the indices.
        instructions = gen_new_index_and_patch_instructions(repodatas[subdir])

        # Step 2c. Output this to $PREFIX so that we bundle the JSON files.
        patch_instructions_path = join(prefix_subdir, "patch_instructions.json")
        with open(patch_instructions_path, "w") as fh:
            json.dump(
                instructions,
                fh,
                indent=2,
                sort_keys=True,
                separators=(",", ": "),
            )


if __name__ == "__main__":
    sys.exit(main())
